/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package filter

import (
	"context"
	"github.com/fonchain_enterprise/micro-account/pkg/e"
	perrors "github.com/pkg/errors"
	"regexp"
	"strconv"
)

import (
	"dubbo.apache.org/dubbo-go/v3/common/extension"
	"dubbo.apache.org/dubbo-go/v3/filter"
	"dubbo.apache.org/dubbo-go/v3/protocol"
)

type validator interface {
	Validate() error
}

func init() {
	extension.SetFilter("fonValidateFilter", NewFonValidateFilter)
}

func NewFonValidateFilter() filter.Filter {
	return &FonValidateFilter{}
}

type FonValidateFilter struct {
}

func (f *FonValidateFilter) Invoke(ctx context.Context, invoker protocol.Invoker, invocation protocol.Invocation) protocol.Result {
	if len(invocation.Arguments()) > 0 {
		if v, ok := invocation.Arguments()[0].(validator); ok {
			if err := v.Validate(); err != nil {
				errMsg := err.Error()
				re3, _ := regexp.Compile(`^invalid(.*): `)
				rep := re3.ReplaceAllString(errMsg, "")
				if errCode, err := strconv.ParseInt(rep, 10, 64); err == nil {
					return &protocol.RPCResult{Err: perrors.Errorf("%v", e.GetMsg(int(errCode)))}
				}

				return &protocol.RPCResult{Err: perrors.Errorf("%v", rep)}
			}
		}
	}

	return invoker.Invoke(ctx, invocation)
}

func (f *FonValidateFilter) OnResponse(ctx context.Context, result protocol.Result, invoker protocol.Invoker, protocol protocol.Invocation) protocol.Result {
	return result
}
